﻿using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using CharMY.Collections.Generic;
using CharMY.IO;
using CharMY.Reflection;
using CharMY.Windows.Forms;

namespace CharMY.Sample
{
    //CSVの入出力

    partial class Sample
    {
        [STAThread]
        public static void Main()
        {
            Consoles.Initialize();

            LinqToCsv.__7つのコーディング方法の比較__();

            Console.ReadLine();
        }
    }

    public static partial class LinqToCsv
    {
        public static void __7つのコーディング方法の比較__()
        {
            // 難しさ：コードの生成・理解に必要な知識量に基づいた指標
            // ↑易
            _6_ForLoopToSimpleCsv();
            _5_HybridLinqToSimpleCsv();
            _4_LinqToSimpleCsv();
            _7_LinqToSimpleCsvDotNETOriginal();
            _3_LinqToCsvVariables();
            _2_LinqToCsvAsDynamic();
            _1_LinqToCsv();
            // ↓難　▼▼▼▼▼


            // 便利さ：コード生成に必要なコストの指標（≒処理内容の理解しやすさ）
            // ↑便利　★★★★★
            _2_LinqToCsvAsDynamic();
            _1_LinqToCsv();
            _3_LinqToCsvVariables();
            _4_LinqToSimpleCsv();
            _5_HybridLinqToSimpleCsv();
            _7_LinqToSimpleCsvDotNETOriginal();
            _6_ForLoopToSimpleCsv();
            // ↓面倒


            // パフォーマンス：ビックデータ処理速度の指標
            // ↑速　●●●●●
            _4_LinqToSimpleCsv();
            _1_LinqToCsv();
            _3_LinqToCsvVariables();
            _7_LinqToSimpleCsvDotNETOriginal();
            _2_LinqToCsvAsDynamic();
            _5_HybridLinqToSimpleCsv();
            _6_ForLoopToSimpleCsv();
            // ↓遅
        }

        // 読込 + 編集 + 出力

        /// <summary>
        /// １ステートメントでCSV処理;
        /// <para>　難しさ　：▼▼▼▼</para>
        /// <para>　便利さ　：★★★★☆</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：●●●●○</para>
        /// <para>LIGHT DATA：●●○</para>
        /// </summary>
        public static void _1_LinqToCsv()
        {
            // ファイル形式
            Files.Csv()
                // ファイルを選択して読込, ヘッダ（ラベル等）とデータの関連付け
                //（第2引数format値はデータ開始行, 第2引数format値-1がラベル行）
                //（第2引数format省略で自動判定）
                // 第2引数format値 = 1 なのでヘッダとしてラベル行のみを使用
                .LoadRecords(format: 1, checkHeaders: true)
                // 読込成功の確認
                .Cache().IfNotNull(records => records
                    // 一行ごとの構造体に変換
                    .Select(Record => new // MyRecordClass() // SampleLinqAsDynamic()で自動生成されたクラスコードを読み込んで利用できます
                    {
                        // ファイルの内容に応じて作成

                        // ラベルが "ID" の列の整数値を取得（例外値は-1）
                        // 新規IDとしてデータを格納
                        NewID = Record.Get("ID", -1) + 10000,
                        // ラベルが "Value" の列の実数値を取得（例外値はNaN）
                        Value = Record.Get("Value", double.NaN),

                        Record
                    })


                    // **** LINQ 処理 (フィルタ) ****
                    .Where(record => record.Value > 0)


                    // 出力テーブル（行×列）の作成（format: 出力ヘッダ数を指定）
                    .ToTextRecordsBuilder(5)
                    // 元のデータ列を追加
                    .AddRecord(records, record => record.Record)
                    // 新規データ列（NewID）を追加
                    .Add(record => record.NewID)
                    // ファイルの末尾に-editedを付加して保存
                    .Save(file: records.MainFile.ChangeName((name, ext) => name + "-edited" + ext), checkOutput: 10)
                );
        }

        /// <summary>
        /// コードの簡便さを追求！
        /// <para>　難しさ　：▼▼▼▽▽</para>
        /// <para>　便利さ　：★★★★☆</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：●●●○</para>
        /// <para>LIGHT DATA：●○○</para>
        /// </summary>
        public static void _2_LinqToCsvAsDynamic()
        {
            // ファイル形式
            Files.Csv()
                // ファイルを選択して読込, ヘッダ（ラベル等）とデータの関連付け
                // ヘッダ自動判定，ヘッダタイプ自動変換 (閾値 0.3)
                .LoadRecords(checkTypeValidRatio: 0.3, checkHeaders: true)
                // 読込成功の確認
                .Cache().IfNotNull(records => records
                    // 一行ごとのオブジェクトに変換, 読込用C#コードを自動生成
                    .ToObjects(outputCode: records.MainFile.ChangeName("MyRecordClass.cs"))


                    // **** LINQ 処理 (フィルタ) ****
                    .Where(record => record.Value > 0)


                    // 出力テーブル（行×列）の作成
                    .ToTextRecordsBuilder(records.Format)
                    // 新規データ列（NewID）を追加
                    .Add<int>("NewID", record => record.ID + 10000)
                    // 元のデータ列を追加
                    .AddRecord(records, record => record.Record)
                    // 出力確認 10 行, 名前をつけてCSVファイルとして保存
                    .Save(checkOutput: 10)
                );
        }
        [SampleCode]
        public static void _2_LinqToCsvAsDynamic_02()
        {
            var records = Files.Csv().LoadRecords();
            records.ToObjects()

                .Where(record => record.Value > 0)

                .ToTextRecordsBuilder(records.Format)
                .AddRecord(records, record => record.Record)
                .Add("NewID", record => record.ID + 10000)
                .Save();
        }

        // 読込 → records → 編集 → table → 出力

        /// <summary>
        /// 変数に途中処理を格納しながらのコーディング
        /// <para>　難しさ　：▼▼▼▽</para>
        /// <para>　便利さ　：★★★★</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：●●●●○</para>
        /// <para>LIGHT DATA：●●○</para>
        /// </summary>
        public static void _3_LinqToCsvVariables()
        {
            // 複数ファイルの選択，読込，ヘッダ（ラベル等）とデータの関連付け
            //（第2引数format値はデータ開始行, 第2引数format値-1がラベル行）
            //（第2引数format省略で自動判定）
            var records = __io.LoadRecords(checkHeaders: true);

            var table = records
                // 一行ごとの構造体に変換
                .Select(Record => new // MyRecordClass() // 自動生成コードに対応
                {
                    // ファイルの内容に応じて作成

                    // ラベルが "ID" の列の整数値を取得（例外値は-1）
                    // 新規IDとしてデータを格納
                    NewID = Record.Get("ID", -1) + 10000,
                    // ラベルが "Value" の列の実数値を取得（例外値はNaN）
                    Value = Record.Get("Value", double.NaN),

                    Record
                })
                // **** LINQ 処理 (フィルタ) ****
                .Where(record => record.Value > 0.0)
                // 出力テーブル（行×列）の作成（format: 出力ヘッダ数を指定）
                .ToTextRecordsBuilder(5)
                // 出力データ列を追加
                .AddRecord(records, record => record.Record)
                // 新規データ列（NewID）を追加
                .Add(record => record.NewID);

            OutputTable(table, records.MainFile);
        }

        /// <summary>
        /// ラベルのない（ラベルを使わない）CSVファイルの操作
        /// <para>　難しさ　：▼▼▽▽</para>
        /// <para>　便利さ　：★★★</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：●●●●○</para>
        /// <para>LIGHT DATA：●●○○</para>
        /// </summary>
        public static void _4_LinqToSimpleCsv()
        {
            var records = GetRecords();

            var table = records
                // **** LINQ 処理 (フィルタ) ****
                .Where(record => record[__index_value].To(0.0) > 0.0)
                // 出力レコードの作成
                .Select(record => record.ConcatItem((record[__index_id].To(0) + 10000).ToString()));

            OutputTable(table);
        }

        /// <summary>
        /// LINQとループを併用したCSVファイルの操作
        /// <para>　難しさ　：▽▽▽</para>
        /// <para>　便利さ　：★★☆</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：○○○</para>
        /// <para>LIGHT DATA：●○○</para>
        /// </summary>
        public static void _5_HybridLinqToSimpleCsv()
        {
            var records = GetRecords();

            // **** 処理 (フィルタ) ****
            records = records
                .Where(record => record[__index_value].To(0.0) <= 0.0);

            // 編集データを一時格納する出力用のコレクションを作成
            var table = new TextRecordsBuilderArranged();
            foreach (var record in records)
            {
                // 出力レコードの作成
                var new_line = record
                    .ConcatItem((object)(record[__index_id].To(0) + 10000));

                table.AddRecord(new_line);
            }

            OutputTable(table);
        }

        /// <summary>
        /// メインの処理にLINQを利用しないCSVファイルの操作
        /// <para>　難しさ　：▽▽</para>
        /// <para>　便利さ　：☆☆☆</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：○○</para>
        /// <para>LIGHT DATA：●○○</para>
        /// </summary>
        public static void _6_ForLoopToSimpleCsv()
        {
            var records = GetRecords();

            // CSV行列の二次元配列を作成
            var matrix = records.ToArrayMatrix();

            // 編集データを一時格納する出力用のコレクションを作成
            var table = new TextRecordsBuilderArranged();
            var table_record = 0;
            for (int row = 0 ; row < matrix.GetLength(0) ; row++)
            {
                // **** 処理 (フィルタ) ****
                if (matrix[row, __index_value].To(0.0) <= 0.0)
                    continue;

                // 出力レコードの作成
                int matrix_max_column = matrix.GetLength(1);
                for (int column = 0 ; column < matrix_max_column ; column++)
                {
                    table[column, table_record] = matrix[row, column];
                }
                table[matrix_max_column, table_record] = matrix[row, __index_id].To(0) + 10000;

                ++table_record;
            }

            OutputTable(table);
        }

        #region 3～6 における共通処理

        // ファイルの内容・項目順に応じて作成
        private const int __index_id = 0;
        private const int __index_value = 1;

        /// <summary>
        /// ファイル形式
        /// </summary>
        private static ITextValueIO __io = Files.Csv().ChangeName("sample");

        /// <summary>
        /// 共通読込処理
        /// </summary>
        /// <returns>読み込んだデータ</returns>
        private static IEnumerable<string[]> GetRecords()
        {
            return __io
                // 読込
                .Load()
                // ラベルなどを読み飛ばすときに
                .Skip(0);
        }

        /// <summary>
        /// 共通出力処理
        /// </summary>
        /// <param name="table">出力するテーブルデータ</param>
        /// <param name="file">出力先ファイル</param>
        private static void OutputTable(IEnumerable<IEnumerable<string>> table, FileInfo file = null)
        {
            // GUI で出力確認!
            table.ShowOnDialog();

            Console.WriteLine("出力確認");
            Consoles.WriteTable(table.Take(10));
            Console.ReadLine();
            Console.WriteLine();

            file = file ?? __io.GetLastAccessFile();
            if (file != null)
            {
                Console.WriteLine("Save to " + file.ToInformation());
                __io.SaveTable(table,
                    file.ChangeName((name, ext) => name + "-edited" + ext));
            }
        }

        #endregion

        // 付録

        /// <summary>
        /// CharMY フレームワークを利用しないCSVファイルの操作
        /// <para>　難しさ　：▼▼▼</para>
        /// <para>　便利さ　：★☆</para>
        /// <para>パフォーマンス</para>
        /// <para> BIG  DATA：●●●○○</para>
        /// <para>LIGHT DATA：●●○○</para>
        /// </summary>
        public static void _7_LinqToSimpleCsvDotNETOriginal()
        {
            #region var data = 読込処理(); <.NET>

            // カレントディレクトリのa.csvを読込
            var records = File
                .ReadLines(Environment.CurrentDirectory + "/a.csv")
                .Select(line => line.Split(','));

            #endregion

            var table = records
                // **** LINQ 処理 (フィルタ) ****
                .Where(record => double.Parse(record[__index_value]) > 0.0)
                // 出力レコードの作成
                .Select(record => record
                    .Concat(new string[]
                    {
                        (int.Parse(record[__index_id]) + 10000).ToString()
                    }));

            #region 出力処理(table); <.NET>

            Console.WriteLine("出力確認");
            foreach (var record in records.Take(10))
                Console.WriteLine(string.Concat(record.Select(item => item + "  ")));
            Console.ReadLine();

            Console.WriteLine("ファイルの末尾に-editedを付加して保存");
            File.WriteAllLines(
                Environment.CurrentDirectory + "/a-edited.csv",
                table.Select(record => record.Aggregate((x, y) => x + "," + y)));

            #endregion
        }
    }
}
