﻿using System;
using System.Linq;
using CharMY.Analysis;
using CharMY.Reflection;

namespace CharMY.Sample
{
    partial class Sample
    {
        //乱数の検定

        [STAThread]
        public static void Main()
        {
            int difLen = 100;
            int take = 200;
            RandomTest(new Random(0), difLen, take);
            RandomTest(new RandomXOr128(), difLen, take);
            RandomTest(new RandomMersenneTwister19937(0), difLen, take);
            RandomTest(new RandomMersenneTwister19937For64Bit(7), difLen, take);
            Console.ReadLine();
        }
        private static void RandomTest(Random rand, int difLen, int take)
        {
            Console.WriteLine("  TYPE = " + rand.GetType().ToTypeString());

            //理論平均0.5からの乖離
            var average = rand.GetDoubles().Take(take).Average();
            Console.WriteLine("AveDif = " + Math.Abs(average - 0.5));

            //d個飛ばしの値の差の平均
            var dif = Enumerable.Range(0, difLen)
                .Select(d => Math.Abs(rand.GetDoubles()
                    .ZipOffset(0, d)
                    .Select(p => p[0] - p[d])
                    .Take(take)
                    .Average())
                )
                .Average();
            Console.WriteLine("DifAve = " + dif);

            var ii = Enumerable.Range(0, difLen).ToArray();
            var s = rand.GetDoubles()
                .ZipOffset(ii)
                .Take(take)
                .CalculateStatisticalValues((v, _, i) => v[i], ii.Length);

            {
                //仮説検定：帰無仮説：平均が0.5ではない
                var p = s.TestDifferenceOfAverageValue(0, 0.5);
                Assert.True(p > 0.05, "Ave=" + p + "  " + s.Average(0) + "  TYPE=" + rand.GetType().ToTypeString());
            }
            for (int i = 1 ; i < s.Dim ; ++i)
            {
                //仮説検定：帰無仮説：個飛ばしの値の差が0ではない
                var p = s.TestDifferenceOfAverage(0, i);
                Assert.True(p > 0.05 / s.Dim, "P=" + p + "  " + s.DifferenceAverage(0, i) + "  TYPE=" + rand.GetType().ToTypeString());
            }

            Console.WriteLine();
        }

    }
}
