﻿using System;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Linq;
using System.Numerics;
using CharMY.Analysis.Graphs;
using CharMY.Analysis.Interfaces;
using CharMY.Drawing;
using CharMY.Reflection;
using CharMY.Windows.Forms;

namespace CharMY.Sample
{
    partial class Sample
    {
        //グラフの描画・最短経路探索
        [STAThread]
        [GuiTest]
        public static void Main()
        {
            var rand = new Random();

            #region create graph

            int graph_size = 50;
            var data = rand
                .GetIntegers(1, graph_size)
                .ZipOffset(0, 1)
                .Where(obj => obj[0] > obj[1])
                .Select(obj => obj[0] + "-" + obj[1])
                .Skipping()
                .Take(graph_size)
                .Distinct()
                .Select(obj => new
                {
                    o = obj.Substring(0, obj.IndexOf("-")),
                    d = obj.Substring(obj.IndexOf("-") + 1),
                    cost = rand.NextDouble() / rand.NextDouble(),
                })
                .SelectMany(obj => new[] { obj, new { o = obj.d, d = obj.o, obj.cost } })
                .ToArray();

            Console.WriteLine(data
                .Select(obj => obj.o + " -> " + obj.d)
                .ToSingleStringLines()
            );

            Console.WriteLine("Node Count = " + data.SelectMany(obj => new[] { obj.o, obj.d }).Distinct().Count());
            Console.WriteLine("Link Count = " + data.Count());

            var graph = data
                .ToGraphSchemeFromEdges(
                    edge => edge.o,
                    edge => edge.d
                )
                .SelectData(_ => _, edge => edge.cost)
                .CreateReadOnlyGraph();

            #endregion

            var sub_list = graph.DivideToNonconnectedSubGraphs();

            //最短経路探索
            var sp = sub_list[0].ShortestPaths(OperationInfoConverter.NoChange<double>(), 0);
            MethodTimer.CheckCount = 1;
            MethodTimer.Invoke("search", () => sp.SearchAllShortestPath());


            //全起終点間の最短経路集合の中から　経路コストが大きなものをとってくる　→　描画時に色を付ける

            Console.WriteLine("max calc cost = 10^" + BigInteger.Log10(CalcCost(graph_size)));
            var paths = sp.AllShortestPaths()
                .OrderBy(path => path.Cost)
                .ToArray();
            var logest_path_edges1 = sp.GetPathEdges(paths[paths.Length - 1]).ToHashSet();
            var logest_path_edges2 = sp.GetPathEdges(paths[paths.Length - 2]).ToHashSet();

            var logest_path_edges3 = sp.GetPathEdges(paths[paths.Length - 5]).ToHashSet();
            var logest_path_edges4 = sp.GetPathEdges(paths[paths.Length - 6]).ToHashSet();

            var use_nodes = sub_list[0].Nodes.ToHashSet();

            #region output graph

            //ノードを適当に配置して描画

            var func = sub_list[0].DrawMethod(
                edgeLength: edge => (float)edge.Data,
                edgeStyleUpdate: (s, edge) =>
                {
                    if (logest_path_edges3.Contains(edge) || logest_path_edges4.Contains(edge))
                        s.Pen.Brush.Color = Color.Orange;

                    if (logest_path_edges1.Contains(edge) || logest_path_edges2.Contains(edge))
                        s.Pen.Brush.Color = Color.Red;

                    s.Text.Font.Size = 8;
                    s.Pen.EndCap = LineCap.AnchorMask;
                },
                nodeText: node => node.Name,
                edgeText: edge => (edge.Data * 100).ToString("F0") + "~",
                bounds: DrawingMaterials.Rectangle(20F, 20F, 400F, 400F),
                randam: rand
            );

            var dialog = Sequence.Loop()
                .Select(_ =>
                {
                    var meta = new MetaImageBuilder();
                    var size = func(meta.Graphics2D);
                    meta.Graphics2D.DrawString("Size = " + size.ToString("E2"), 10F, 10F);
                    return meta;
                })
                .ImagesView(interval: 20, nonTransparent: true);
            dialog.ShowDialog();

            #endregion
        }
        private static BigInteger CalcCost(BigInteger size)
        {
            if (size <= 1)
                return 1;

            var cache = CalcCost(size - 1);
            return cache + cache * size;
        }
    }
}
