﻿using System;
using System.Drawing;
using System.Linq;
using CharMY.Drawing;
using CharMY.Windows.Forms;

namespace CharMY.Sample
{
    partial class Sample
    {
        [STAThread]
        public static void Main()
        {
            //フレームごとのイメージを作成
            var imgs = Sequence.For(0, i => i < 64, i => i + 2)
                .Select(i => BitmapBuilder.FromImage(TestBitmap(i,
                    Color.FromArgb(0xFF - i * 3, i * 3, 0),
                    Color.FromArgb(0xCF - i * 3, 0, i * 3 + 0x40),
                    Color.FromArgb(0, 0xFF - i * 3, i * 3),
                    Color.FromArgb(0xCF - i * 3, 0, i * 3 + 0x40),
                    Color.FromArgb(0)
                )))
                .ToArray();

            //フレームの統合・パラメータの設定
            var range = Enumerable.Range(8, 32).ToArray();
            var args = new ColorTableMergeArguments();
            args.LayeredClusteringMode = false;
            var anm_source = GifAnimationSource.Create(imgs.Concat(imgs.Reverse()), commonColorTable: args, localTableFrame: range);
            anm_source.SetAll(f =>
            {
                f.DelayTimeCentSec = 5;
                f.Optimization = GifData.DisposalMethod.Fill;
            });
            anm_source.ColorTableOptimize(null, strictCommonTable: args, localTableFrame: range);
            anm_source.DisposalOptimize();

            Console.WriteLine("start creation");

            var gif = anm_source.ToGifData();

            //GIFアニメの表示
            var iv = gif.ImageView(nonTransparent: true);
            iv.Text = "size = " + gif.Data.Count().ToString("N0") + " byte";
            iv.ShowDialog();

            //フレーム９の表示
            gif.Frames[9]
                .ToIndexedImage()
                .ImageView()
                .ShowDialog();
        }
        private static Bitmap TestBitmap(int x, Color a1, Color a2, Color b1, Color b2, Color c)
        {
            var bmp = new Bitmap(200, 200);
            var g = Graphics.FromImage(bmp);
            var br_c = new SolidBrush(c);
            g.FillRectangle(br_c, 0, 0, bmp.Width, bmp.Height);

            var br_a = new BrushStyle();
            var br_b = new BrushStyle();
            br_a.Type = BrushStyleType.LinearGradient;
            br_b.Type = BrushStyleType.LinearGradient;
            br_a.Rectangle = new RectangleF(0, 0, bmp.Width / 2, bmp.Height / 2);
            br_b.Rectangle = new RectangleF(0, 0, bmp.Width / 2, bmp.Height / 2);
            br_a.Color = a1;
            br_a.Color2 = a2;
            br_b.Color = b1;
            br_b.Color2 = b2;
            g.FillEllipse(br_b.CreateBrush(), 50, 50, 100 - x / 2, 100);
            g.FillEllipse(br_a.CreateBrush(), x, BitLogic.Sqrt(x * x), 100, 100 - x / 2);
            return bmp;
        }
    }
}
