﻿using System;
using System.Drawing;
using System.Linq;
using CharMY.Analysis;
using CharMY.Data;
using CharMY.Drawing;
using CharMY.Text;
using CharMY.Windows.Forms;

namespace CharMY.Sample
{
    partial class Sample
    {
        //描画サンプル

        [STAThread]
        public static void Main()
        {
            using (var meta = new MetaImageBuilder())
            {
                var g = meta.Graphics2D;
                var style_a = new FigureStyle();
                var style_b = new FigureStyle();
                var style_c = new FigureStyle();
                style_a.Mode = DrawMode.Closed;
                style_a.Pen.Brush.Color = Color.Violet;
                style_b.Mode = DrawMode.FillAlternate;
                style_b.Brush.Color = Color.Salmon;
                style_c.Area.Size = DrawingMaterials.Size(20F, 20F);
                for (int i = 3 ; i <= 13 ; ++i)
                    for (int j = 1 ; j <= i / 2 ; ++j)
                    {
                        style_c.PlotSimplexCount = i;
                        style_c.PlotSimplexRotation = j;
                        g.DrawPlot(i * 30, j * 30, style_c.OverwritedBy(style_a));

                        g.DrawPlot(i * 30, 200 + j * 30, style_c.OverwritedBy(style_b));
                    }
                meta.ImageView("S A M P L E")
                    .ShowDialog();
            }
        }

        public static DataTable SampleDataSet(int record, int column = 3)
        {
            var rand = new Random();
            var table = new DataTable("sample");
            var columns = Enumerable.Range(0, column)
                .Select(c => table.AddColumn("Data" + c.ToExcelColumnLabel(), double.NaN))
                .ToArray();

            var setters = columns
                .Select(col => col.GetHeader<double>(DataHeadKeys.ValueSetter));

            for (int r = 0 ; r < record ; ++r)
            {
                foreach (var setter in setters)
                    setter.Value = Statistics.NormSInv(rand.NextDouble());

                table.Setter.AddToTable();
            }

            return table;
        }

        public static ChartCreator ChartCreatorSample001(ChartCreator maker, bool count)
        {
            maker.Title = "fig.1 sample";
            maker.Format.Options |= ChartFormatOptions.TitleLocateBottom;
            maker.Format.Options |= ChartFormatOptions.VerticalColorBar;

            var x = maker.Axes.X();
            x.Label = "a";
            x.MaximumValue = 3.25;
            x.MinimumValue = -3.25;
            x.Format.NumberFormat = NumberFormats.DefaultFloat;

            var y = maker.Axes.Y();
            y.Label = "b";
            y.MaximumValue = 3.25;
            y.MinimumValue = -3.25;
            y.Format.NumberFormat = NumberFormats.DefaultFloat;

            if (!count)
            {
                maker.Format.PlotColoring.ErrorColor = Color.BlueViolet;
                maker.Axes.PlotWidth().Label = "";
                maker.Axes.PlotHeight().Label = "";
                var z = maker.Axes.Z();
                z.Label = "c";
                z.MaximumValue = 5;
                z.MinimumValue = -5;
                z.Format.NumberFormat = NumberFormats.DefaultDouble;

                maker.Axes.SubColor().MinimumValue = -20;
                //maker.AxesForUpdate.SubColor().Options = ChartAxialValueOptions.ReverseValue;
                //maker.AxesForUpdate.SubColor().CalculationOptions = ComponentModel.OperationDefaultUnarys.Ln;
                //maker.AxesForUpdate.SubColor().IntervalLabel = 2.0;
            }

            return maker;
        }
    }
}
